% My_fill_cubicgrid
% Y. Yang, UCLA Physics & Astronomy
% First version date: 2015. 04. 30.
% output parameter: Proj (a 2D projection image from 3D volume along the
%                         given Euler angles)
% input parameter: Vol (input 3D volume)   
%                  phi (Phi Euler angle)                  
%                  theta (Theta Euler angle)
%                  psi (Psi Euler angle)
%                  varargin (optional argument for cut size of the volume
%                  to avoid out of memory problem during rotation of big
%                  volume)


function Proj = My_get_3D_projection_interp(Vol, phi, theta, psi, varargin)

    Vol = permute(Vol, [2 1 3]);

    % calculate rotation matrix
    vector3 = [0 0 1];
    rotmat3 = MatrixQuaternionRot(vector3,phi);    

    vector2 = [0 1 0];
    rotmat2 = MatrixQuaternionRot(vector2,theta);

    vector1 = [0 0 1];
    rotmat1 = MatrixQuaternionRot(vector1,psi);

    rotMATs =  (rotmat1*rotmat2*rotmat3);
    
    % pixel positions
    Yarr = My_find_symm_indarr(size(Vol,1));
    Xarr = My_find_symm_indarr(size(Vol,2));
    Zarr = My_find_symm_indarr(size(Vol,3));

    [X, Y, Z] = meshgrid(Xarr, Yarr, Zarr);

    Xshape = size(X);
    Yshape = size(Y);
    Zshape = size(Z);

    % rotated pixel positions
    posvec = [X(:), Y(:), Z(:)]';
    rot_posvec = rotMATs*posvec;

    rotX = reshape(rot_posvec(1,:),Xshape);
    rotY = reshape(rot_posvec(2,:),Yshape);
    rotZ = reshape(rot_posvec(3,:),Zshape);

    clear posvec;
    clear rot_posvec;

    % cut the 3D volume for rotation interpolation to avoid out of memory
    % problem
    if nargin > 4
        cut_size = varargin{1};
    else
        cut_size = 50;
    end

    cutnum = floor(size(rotX,3)/cut_size);
    cutrem = mod(size(rotX,3),cut_size);

    ROTvol = zeros(size(rotX));
    
    % run 3D interpolation for rotation
    for i=1:cutnum
        cutindar = ((i-1)*cut_size+1):i*cut_size;
        ROTtemp = interp3(X,Y,Z,Vol,rotX(:,:,cutindar),rotY(:,:,cutindar),rotZ(:,:,cutindar),'linear',0);
        ROTvol(:,:,cutindar) = ROTtemp;    
    end

    if cutrem~=0
        cutindar = (cutnum*cut_size+1):size(rotX,3);
        ROTtemp = interp3(X,Y,Z,Vol,rotX(:,:,cutindar),rotY(:,:,cutindar),rotZ(:,:,cutindar),'linear',0);
        ROTvol(:,:,cutindar) = ROTtemp;
    end

    % obtain 2D projection
    ROTvol = permute(ROTvol, [2 1 3]);
        
    Proj = squeeze(sum(ROTvol,2))';
    
end