function Err = My_proj_compare(proj1,proj2)
    % Otsu treshold parameter (the smaller the parameter, the more generous the
    % mask, i.e., larger mask)
    th_dis_r_afterav = 0.10;

    % gaussian smoothing kernel size for final masking after averaging
    avg_bg_conv_size = 5;

    % smoothing convolution masking threshold
    conv_threshold = 0.1;

    % masking final averaged image
    A1=edge_preserve_smoothing_YY(proj2,1,1);

    % Otsu threshold
    im=double(A1/max(max(A1))*255);
    [ot,outim]=otsu_thresh(im);
    ot=ot*max(max(A1))/255*th_dis_r_afterav;

    after_conv = (A1>ot) * 1;
    
    % convolute the mask with given size
    after_conv = bg_conv_gauss(after_conv, avg_bg_conv_size, conv_threshold);
    
    fbg_mask = ~after_conv;

    % run background subtraction using the convoluted mask
    bgsub_image = proj2;
    bgsub_image(fbg_mask) = 0;   

    [comX1, comY1] = My_COM_2D(proj1);
    [comX2, comY2] = My_COM_2D(bgsub_image);
    
    comXdiff = comX1-comX2;
    comYdiff = comY1-comY2;
    
    if comXdiff > 2 || comYdiff > 2
        'too big difference'
    end
    
    shift_proj2 = real(My_FourierShift(bgsub_image,comXdiff,comYdiff));
    
    shift_proj2(fbg_mask) = 0;
    
    %[comX2p, comY2p] = My_COM_2D(shift_proj2);
    
    %fXdiff = comX1-comX2p
    %fYdiff = comY1-comY2p
    
    Err = calcR_norm_YY(proj1,shift_proj2);
    
end
    
    
    