% Orient_cubic_crystal_new_yxz_pad_three_twoalso_YY
% Y. Yang, UCLA Physics & Astronomy
% First version date: 2015. 07. 13.

% output parameter: U: change-of-basis matrix for [100] orientation
% input parameters: ESTvol:  3D reconstruction volume
%                   res: pixel size (resolution)0.3725, 1.93, 0.02, 5,8,1,1
%                   lattice_const: lattice constant of the cubic crystal
%                   mag_tolerance: tolerance range for q magnitude of the
%                                  Fourier peaks
%                   ang_tolerance: tolence range from 90 deg for angle search
%                   rankmax: maximum # of peaks to find
%                   userank: minumum rank of peaks to use for matrix search
%                   padnum: oversampling ratio for padding

% Orient_cubic_crystal_new_yxz_pad_three_twoalso_YY finds the
% change-of-basis matrix to transform 3D reconstruction volume of a cubic
% crystal to make the {100} directions along the MATLAB array directions.

function U = Orient_cubic_crystal_new_yxz_pad_three_twoalso_YY(ESTvol,res,lattice_const,mag_tolerance,ang_tolerance,rankmax,userank,padnum)

    cenar_1 = My_get_center_ind(size(ESTvol,1)*padnum,size(ESTvol,1));
    cenar_2 = My_get_center_ind(size(ESTvol,2)*padnum,size(ESTvol,2));
    cenar_3 = My_get_center_ind(size(ESTvol,3)*padnum,size(ESTvol,3));

    ESTvol_padded = zeros(size(ESTvol,1)*padnum,size(ESTvol,2)*padnum,size(ESTvol,3)*padnum);
    ESTvol_padded(cenar_1,cenar_2,cenar_3) = (ESTvol);
    
    KKK = abs(fftshift(fftn(ifftshift(ESTvol_padded))));
    
    clear ESTvol_padded;
    
    %KKK = smooth3(KKK,'gaussian',5);
    
    d001 = (1/lattice_const);
    
    locmax = KKK > circshift(KKK,1,1) & KKK > circshift(KKK,-1,1) & ...
             KKK > circshift(KKK,1,2) & KKK > circshift(KKK,-1,2) & ...
             KKK > circshift(KKK,1,3) & KKK > circshift(KKK,-1,3) & ...
             KKK > circshift(KKK,[1 1 0]) & KKK > circshift(KKK,[1 -1 0]) & ...
             KKK > circshift(KKK,[-1 1 0]) & KKK > circshift(KKK,[-1 -1 0]) & ...
             KKK > circshift(KKK,[0 1 1]) & KKK > circshift(KKK,[0 1 -1]) & ...
             KKK > circshift(KKK,[0 -1 1]) & KKK > circshift(KKK,[0 -1 -1]) & ...
             KKK > circshift(KKK,[1 0 1]) & KKK > circshift(KKK,[1 0 -1]) & ...
             KKK > circshift(KKK,[-1 0 1]) & KKK > circshift(KKK,[-1 0 -1]) & ...
             KKK > circshift(KKK,[1 1 1]) & KKK > circshift(KKK,[1 1 -1]) & ...
             KKK > circshift(KKK,[1 -1 1]) & KKK > circshift(KKK,[1 -1 -1]) & ...
             KKK > circshift(KKK,[-1 1 1]) & KKK > circshift(KKK,[1 1 -1]) & ...
             KKK > circshift(KKK,[-1 -1 1]) & KKK > circshift(KKK,[-1 -1 -1]);
    
    Yarr = My_find_symm_indarr(size(KKK,1))*(1/res/size(KKK,1));
    Xarr = My_find_symm_indarr(size(KKK,1))*(1/res/size(KKK,2));
    Zarr = My_find_symm_indarr(size(KKK,1))*(1/res/size(KKK,3));
        
    [X, Y, Z] = meshgrid(Xarr, Yarr, Zarr);    
    
    KKKmag = sqrt(X.^2+Y.^2+Z.^2);
    
    clear X;
    clear Y;
    clear Z;
    
    KKKmag(~locmax) = 0;
    
    clear locmac;
    
    KKKmag(KKKmag < (d001-mag_tolerance) | KKKmag > (d001+mag_tolerance)) = 0;
    
    KKKmag_long = KKKmag(:);
    
    KKKint_long = abs(KKK(:));
    
    KKKmag_long(KKKmag == 0) = [];
    
    KKKint_long(KKKmag==0) = [];
    
    clear KKKmag;
    
    if length(KKKmag_long) < 2;
        fprintf(1,'Not able to find at least two peaks within the desired tolerance\n');
        return
    end
    
    KKKint_sort = sort(KKKint_long, 'descend');
    
    %KKKmag_long = KKKmag(:);
    KKKint_long =abs(KKK(:));
    
    maxind = find(KKKint_long == KKKint_sort(userank),1);
    
    [mYind, mXind, mZind] = My_find_3D_ind(maxind, size(KKK,1), size(KKK,2), size(KKK,3));
    
    mx = Xarr(mXind);
    my = Yarr(mYind);
    mz = Zarr(mZind);
    
    found_int = [];
    found_rank = [];
    found_num = 0;
    smx = []; smy = []; smz = [];
    angle_err = [];
    for i=(userank+1):min(rankmax,length(KKKint_sort))
        currind = find(KKKint_long == KKKint_sort(i),1);
        [cYind, cXind, cZind] = My_find_3D_ind(currind, size(KKK,1), size(KKK,2), size(KKK,3));
        cx = Xarr(cXind);
        cy = Yarr(cYind);
        cz = Zarr(cZind);       
     
        ang = My_find_angle_twovector([mx my mz], [cx cy cz]);
        
        fprintf(1, 'rank %d angle %5.2f norm %6.3e intensity %6.3e \n',i, ang, norm([cx cy cz]), KKKint_sort(i));
        
        if ang < 90 + ang_tolerance && ang > 90 - ang_tolerance
            fprintf(1, 'found a peak of rank %d of angle %5.2f norm %6.3e within the angle tolerance from first peak\n',i, ang, norm([cx cy cz]));
            found_num = found_num + 1;
            found_int(found_num) = KKKint_sort(i);
            found_rank(found_num) = i;
            smx(found_num) = cx; smy(found_num) = cy; smz(found_num) = cz;
            angle_err(found_num) = ang-90;
        end
    end
    
    if found_num == 0
        fprintf(1, 'not able to find second peak within the angle tolerance\n');
    end
  
    min_ind = find(abs(angle_err) == min(abs(angle_err)),1);
    
    min_int = found_int(min_ind);
    min_rank = found_rank(min_ind);
    min_err = angle_err(min_ind);
    
    fprintf(1, 'min_rank: %d, min_angle_err: %5.2f\n',min_rank, min_err);
    
    m_smx = smx(min_ind); m_smy = smy(min_ind); m_smz = smz(min_ind);
    
    vec1 = [mx my mz];
    vec2 = [m_smx m_smy m_smz];
    
    t_found_int = [];
    t_found_rank = [];
    t_found_num = 0;
    t_smx = []; t_smy = []; t_smz = [];
    t_angle_err = [];
    for i=1:length(smx)
        cx = smx(i); cy=smy(i); cz=smz(i);
        ang = My_find_angle_twovector([m_smx m_smy m_smz], [cx cy cz]);
    
        if ang < 90 + ang_tolerance && ang > 90 - ang_tolerance
            fprintf(1, 'found a peak of rank %d of second angle %5.2f norm %6.3e within the angle tolerance from first peak\n',found_rank(i), ang, norm([cx cy cz]));
            t_found_num = t_found_num + 1;
            t_found_int(t_found_num) = KKKint_sort(i);
            t_found_rank(t_found_num) = i;
            t_smx(t_found_num) = cx; t_smy(t_found_num) = cy; t_smz(t_found_num) = cz;
            t_angle_err(t_found_num) = ang-90;
        end
    end
    

    
    t_min_ind = find(abs(t_angle_err) == min(abs(t_angle_err)),1);
    
    t_min_int = found_int(t_min_ind);
    t_min_rank = found_rank(t_min_ind);
    t_min_err = angle_err(t_min_ind);
    
    fprintf(1, '3rd min_rank: %d, min_angle_err: %5.2f\n',t_min_rank, t_min_err);
    
    t_m_smx = t_smx(t_min_ind); t_m_smy = t_smy(t_min_ind); t_m_smz = t_smz(t_min_ind);
       
    vec3 = [t_m_smx t_m_smy t_m_smz];   
    
    if t_found_num == 0
        fprintf(1, 'not able to find third peak within the angle tolerance\n');
        
        Input = input('Do you wish to calculate the matrix only with two peaks? [Y/N]','s');
        if Input =='Y' || Input == 'y'
            vec3 = cross(vec1,vec2);
        end
    end
           
    fprintf(1, 'norm max: %6.3e, norm 2max: %6.3e, norm 3max: %6.3e\n',norm(vec1), norm(vec2), norm(vec3));
    
    n_vec1 = vec1/norm(vec1);
    n_vec2 = vec2/norm(vec2);   
    n_vec3 = vec3/norm(vec3);
    
    if My_find_angle_twovector(cross(n_vec1,n_vec2),n_vec3) > 170
        n_vec3 = -1*n_vec3;
    elseif My_find_angle_twovector(cross(n_vec1,n_vec2),n_vec3) > 10
        fprintf(1, 'something wrong, third angle not orthogonal to first and second\n');
    end
        
    U = [n_vec1', n_vec2', n_vec3'];
    
end
    
    
    