recvolstr = importdata('FePt_final_reconstruction_volume_unrotated.mat');

ESTvol = recvolstr.GENFIRE_rec;


ESTvol_size1 = size(ESTvol,1);
ESTvol_size2 = size(ESTvol,2);
ESTvol_size3 = size(ESTvol,3);

Yarr = My_find_symm_indarr(ESTvol_size1);
Xarr = My_find_symm_indarr(ESTvol_size2);
Zarr = My_find_symm_indarr(ESTvol_size3);

[X, Y, Z] = meshgrid(Xarr, Yarr, Zarr);

% obtain rotation matrix from the {200} peaks of FePt reconstruction volume
% reciprocal space
U = Orient_cubic_crystal_new_yxz_pad_three_twoalso_YY(ESTvol, 0.3725, 1.93, 0.02, 5,8,1,1);


%% 
invU = inv(U);

% take only rotation part of the orientation matrix
[U1, S, U2] = svd(U);
Up = U1*U2';

[X, Y, Z] = meshgrid(Xarr, Yarr, Zarr);

Xshape = size(X);
Yshape = size(Y);
Zshape = size(Z);

% rotate the position vectors
posvec = [X(:), Y(:), Z(:)]';
rot_posvec = Up*posvec;

rotX = reshape(rot_posvec(1,:),Xshape);
rotY = reshape(rot_posvec(2,:),Yshape);
rotZ = reshape(rot_posvec(3,:),Zshape);

clear posvec;
clear rot_posvec;

% cut the reconstruction volume with "cut-size" to run the interpolation
% without hitting the memory limit
cut_size = 50;

cutnum = floor(size(rotX,3)/cut_size);
cutrem = mod(size(rotX,3),cut_size);

% run interpolation for volume rotation
ROTvol = zeros(size(rotX));
for i=1:cutnum
    cutindar = ((i-1)*50+1):i*50;
    ROTtemp = interp3(X,Y,Z,ESTvol,rotX(:,:,cutindar),rotY(:,:,cutindar),rotZ(:,:,cutindar));
    %ROTtemp = interp3(X,Y,Z,Support,rotX(:,:,cutindar),rotY(:,:,cutindar),rotZ(:,:,cutindar));
    ROTvol(:,:,cutindar) = ROTtemp;    
end

if cutrem~=0
    cutindar = (cutnum*50+1):size(rotX,3);
    ROTtemp = interp3(X,Y,Z,ESTvol,rotX(:,:,cutindar),rotY(:,:,cutindar),rotZ(:,:,cutindar));
    %ROTtemp = interp3(X,Y,Z,Support,rotX(:,:,cutindar),rotY(:,:,cutindar),rotZ(:,:,cutindar));
    ROTvol(:,:,cutindar) = ROTtemp;
end



%% make it consistent with other reconstructions
ESTvol = ROTvol;
ESTvol(isnan(ESTvol)) = 0;

ESTvol = flipud(ESTvol);
ESTvol = permute(ESTvol,[3 2 1]);
ESTvol = rot90(ESTvol,2);

%% save result
save FePt_final_reconstruction_volume.mat ESTvol
