

#include <vector>
#include <iostream>
#include "mex.h"
#include <iomanip>
#include <complex>
#include <cmath>
#include <math.h>
using namespace std;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{  // initialize variables/pointers
	double *multiInd;
	double *masterDistances, *masterValuesReal, *masterValuesComplex,*uniqueInd;
	mxArray *MI, *MD, *MVR, *UI;

	//get pointers to input objects
	 multiInd = mxGetPr(prhs[0]);
	 masterDistances = mxGetPr(prhs[1]);
	 masterValuesReal = mxGetPr(prhs[2]);
	 masterValuesComplex = mxGetPi(prhs[2]);
	 uniqueInd = mxGetPr(prhs[3]);
    int numInd, dimy, numdims;
	const mwSize *dims;
	dims = mxGetDimensions(prhs[0]);
	dimy = (int)dims[0];
	numInd = (int)dims[1];

	//initialize outputs
	plhs[0] = mxCreateDoubleMatrix(dimy, numInd, mxREAL);
	plhs[1] = mxCreateDoubleMatrix(dimy, numInd, mxREAL);
	plhs[2] = mxCreateDoubleMatrix(dimy, numInd, mxREAL);

	//I choose to separate real and complex parts and recombine later
	double *magnitudesOut;
	magnitudesOut = mxGetPr(plhs[0]);
	double *valuesOutReal, *valuesOutComplex;
	valuesOutReal = mxGetPr(plhs[1]);
	valuesOutComplex = mxGetPr(plhs[2]);
	vector<double> tmpValuesReal;
	vector<double> tmpValuesComplex;
	vector<double> tmpWeights;

	// loop over each voxel that was repeatedly matched
	for (int i = 0; i <= (numInd - 2); i++){
		
		double ind1 = uniqueInd[(int)multiInd[i] - 1];
		double ind2 = uniqueInd[(int)multiInd[i]]-1;
		// find number of repeated matches
		int lengthOfVector = (int)ind2 - (int)ind1 + 1;
		// resize vectors
		tmpValuesReal.resize(lengthOfVector );
		tmpValuesComplex.resize(lengthOfVector );
		tmpWeights.resize(lengthOfVector );

		double distanceSum;

		distanceSum = 0;
		// compute sum of inverse distances for normalization
		for (int j = 0; j <= lengthOfVector-2; j++){

			tmpValuesReal[j] = masterValuesReal[(int)ind1 + j];
			tmpValuesComplex[j] = masterValuesComplex[(int)ind1 + j];
			distanceSum = distanceSum + 1 / (masterDistances[(int)ind1 + j] + 0.0000000001);
		}
		valuesOutReal[i] = 0; 
		magnitudesOut[i] = 0;
		valuesOutComplex[i] = 0;
		for (int j = 0; j <= lengthOfVector-2; j++){
			tmpWeights[j] = 1 / (masterDistances[(int)ind1 + j] + 0.0000000001) / distanceSum;//current weight
			valuesOutReal[i] = valuesOutReal[i] + tmpWeights[j] * tmpValuesReal[j];//weighted value for real and complex part is used for phase of final value
			valuesOutComplex[i] = valuesOutComplex[i] + tmpWeights[j] * tmpValuesComplex[j];
			magnitudesOut[i] = magnitudesOut[i] + tmpWeights[j] * sqrt(tmpValuesReal[j] * tmpValuesReal[j] + tmpValuesComplex[j] * tmpValuesComplex[j]);//weighted magnitude will be final magnitude
		}
	}

		return;
}

